package com.bank.it.payees.service;

import com.bank.it.payees.constants.APIMessagesConstants;
import com.bank.it.payees.dto.mapper.PayeeMapper;
import com.bank.it.payees.dto.response.PayeeResponse;
import com.bank.it.payees.entity.PayeeEntity;
import com.bank.it.payees.exception.custom.PayeeNotFoundException;
import com.bank.it.payees.kafka.Producer.Producer;
import com.bank.it.payees.model.Payee;
import com.bank.it.payees.repository.IManagePayeeRepository;
import com.bank.it.payees.util.BankAccountNumberGeneratorUtil;
import org.springframework.beans.factory.annotation.Autowired;
import com.bank.it.payees.dto.request.PayeeRequest;
import org.springframework.stereotype.Service;


import java.util.List;
import java.util.Optional;

@Service
public class ManagePayeeServiceImpl implements IManagePayeeService {

    @Autowired
    private IManagePayeeRepository managePayeeRepo;

    @Autowired
    private BankAccountNumberGeneratorUtil bankAccountNumberGeneratorUtil;

    /* @Autowired
     private PayeeKafkaProducer payeeKafkaProducer;
 */
    @Autowired
    private Producer producer;

    /**
     * Persist payee into NOSQL DB
     *
     * @param payeeRequest
     * @return PayeeResponse
     */
    @Override
    public PayeeResponse savePayee(PayeeRequest payeeRequest) {

        payeeRequest.setAccountNumber(bankAccountNumberGeneratorUtil.generatePayeeAccNumber());

        PayeeEntity payeeEntity = PayeeMapper.MAPPER.fromRequestToEntity(payeeRequest);
        PayeeEntity payeeEntityFromDB = managePayeeRepo.save(payeeEntity);

        return PayeeMapper.MAPPER.fromEntityToResponse(payeeEntityFromDB);

    }

    /**
     * Update payee into NOSQL DB
     *
     * @param payeeRequest
     * @return PayeeResponse
     */
    @Override
    public PayeeResponse updatePayee(PayeeRequest payeeRequest) {
        String existingDocumentId = null;

        Optional<PayeeEntity> existingDocumentFromDB = managePayeeRepo.getPayeeEntityByAccountNumber(payeeRequest.getAccountNumber());
        existingDocumentFromDB.orElseThrow(RuntimeException::new);

        payeeRequest.setId(existingDocumentFromDB.isPresent() ? existingDocumentFromDB.get().getId() : existingDocumentId);

        PayeeEntity payeeEntity = PayeeMapper.MAPPER.fromRequestToEntity(payeeRequest);
        PayeeEntity payeeEntityFromDB = managePayeeRepo.save(payeeEntity);

        PayeeResponse payeeResponse = PayeeMapper.MAPPER.fromEntityToResponse(payeeEntityFromDB);

        Payee payee = PayeeMapper.MAPPER.fromEntityToPayee(payeeEntity);
        producer.sendMessage(payee);

        return payeeResponse;

    }

    /**
     * Retrieve payee from NOSQL DB based on accountNumber
     *
     * @param accountNumber
     * @return PayeeResponse
     */
    @Override
    public PayeeResponse searchPayee(int accountNumber) {
        PayeeResponse payeeResponse = null;
        Optional<PayeeEntity> payeeEntity = managePayeeRepo.getPayeeEntityByAccountNumber(accountNumber);
        if(!payeeEntity.isPresent()) throw new PayeeNotFoundException(APIMessagesConstants.PAYEE_NOT_EXIST_SEARCH_MSG);

        payeeResponse = PayeeMapper.MAPPER.fromEntityToResponse(payeeEntity.get());

        return Optional.ofNullable(payeeResponse).orElseThrow(RuntimeException::new);
    }

    /**
     * Retrieve all payees from NOSQL DB
     *
     * @param
     * @return List of PayeeResponse
     */
    @Override
    public List<PayeeResponse> searchPayees() {
        List<PayeeEntity> listOfPayees = managePayeeRepo.findAll();
        if(!(listOfPayees.size()>0)) throw new PayeeNotFoundException(APIMessagesConstants.PAYEE_NOT_EXIST_SEARCH_ALL_MSG);
        return PayeeMapper.MAPPER.fromEntityListToResponseList(listOfPayees);

    }

    /**
     * delete  payee from NOSQL DB
     *
     * @param
     * @return String msg
     */
    @Override
    public String deletePayee(int accountNumber) {

        Optional<PayeeEntity> payeeEntity = managePayeeRepo.getPayeeEntityByAccountNumber(accountNumber);
        if(!payeeEntity.isPresent()) throw new PayeeNotFoundException(APIMessagesConstants.PAYEE_NOT_EXIST_DEL_MSG);
        managePayeeRepo.delete(payeeEntity.get());

        return APIMessagesConstants.PAYEE_DEL_SUCCESS;
    }
}
